#import "BasicMuncher.h"
#import "consumption.h"
#import "PostTracker.h"

#define kFullTableWidth "width=\"97%\" style=\"padding-left: 10px; padding-right: 10px;\""
#define kMaxNonSpaceChars 40

const char *staticScript = "<script>var hnWebPosts = []; var hnWebThreadId = 0; var hnWebRPoint = 0;\n"
"function hnWebInformRead(postId) {\n"
"  var req = new XMLHttpRequest();\n"
"  req.open('GET', 'hn://news.ycombinator.com/hnweb.post.viewed?' + hnWebThreadId + '_' + postId, true);\n"
"  req.send(null);\n"
"}\n"
"function hnWebNodeY(node) { var y = 0;\n"
"  do { y += node.offsetTop; } while (node = node.offsetParent); return y;\n"
"}\n"
"function hnWebCheck() {\n"
"  var mark = window.scrollY + window.innerHeight;\n"
"  while (hnWebRPoint < hnWebPosts.length) {\n"
"    var postId = hnWebPosts[hnWebRPoint];\n"
"    var node = document.getElementById('hnWebPost_' + postId);\n"
"    if (node.offsetHeight + hnWebNodeY(node) >= mark) { break; }\n"
"    hnWebRPoint += 1;\n"
"    hnWebInformRead(postId);\n"
"  }\n"
"}\n"
"window.onscroll = hnWebCheck;\n"
"window.onresize = hnWebCheck;\n"
"window.onload = hnWebCheck;\n"
"</script>\n";

@implementation BasicMuncher

- (BasicMuncher *)init {
	if (self = [super init]) {
		presumedThreadId = nil;
		currentCommentId = nil;
	}
	return self;
}

- (void)reset {
	state = ParseStateTop;
}

/**
 Consumes data from [*posRef, end), advancing *posRef along the way, and dumping
 skipped and filtered data to output.

 This is fragile, but the worst thing you'll do is break things.
 */
- (void)consume:(NSMutableData *)output from:(const char **)posRef to:(const char *)end {
	// r is the protagonist and pos lags behind.  The last thing we do in this
	// function is copy [pos, r) to output.  The purpose of pos is to keep track
	// of data in the interval [pos, r) which has not yet been copied to output.
	// This lets us avoid having to copy lots of tiny small regions in many
	// separate function calls as we advance r through the buffer.
	const char *pos = *posRef;
	const char *r = pos;
more:
	switch (state) {
		case ParseStateTop: {
			const char *segBeg = r;
			const char *segEnd = eatSegment(segBeg, end);
			if (!segEnd) {
				break;
			}
			r = segEnd;
			if (regionEqualsString(segBeg, segEnd, "<link rel=\"stylesheet\" type=\"text/css\" href=\"http://ycombinator.com/news.css\">")) {
				[output appendBytes:pos length:segBeg - pos];
				static const char alternateLink[] = "<link rel=\"stylesheet\" type=\"text/css\" href=\"hn://ycombinator.com/news.css\">";
				[output appendBytes:alternateLink length:strlen(alternateLink)];
				pos = segEnd;
			} else if (regionEqualsString(segBeg, segEnd, "<title>")) {
				[output appendBytes:pos length:segBeg - pos];
				[output appendBytes:staticScript length:strlen(staticScript)];
				[output appendBytes:segBeg length:segEnd - segBeg];
				pos = segEnd;
			} else if (*segBeg != '<') {
				static const char varString[] = "var item = v[1];";
				const char *varFound = findString(segBeg, segEnd, varString);
				if (varFound) {
					const char *endOfVarFound = varFound + strlen(varString);
					[output appendBytes:pos length:endOfVarFound - pos];
					static const char confirmation[] = "if (!confirm('Sure you want to vote ' + v[0] + '?')) { return false; }";
					[output appendBytes:confirmation length:strlen(confirmation)];
					pos = endOfVarFound;
				}
			} else {
				const char *attrBeg = NULL;
				const char *attrEnd = NULL;
				if (isTagWithAttr(segBeg, segEnd, "table", "width=\"85%\"", &attrBeg, &attrEnd)) {
					[output appendBytes:pos length:attrBeg - pos];
					static const char fullWidth[] = kFullTableWidth;
					[output appendBytes:fullWidth length:strlen(fullWidth)];
					[output appendBytes:attrEnd length:segEnd-attrEnd];
					pos = segEnd;
				} else if (regionEqualsString(segBeg, segEnd, "<span class=\"pagetop\">")) {
					state = ParseStateInContentTable;
				}
			}
		}
			goto more;
		case ParseStateInContentTable: {
			const char *segBeg = r;
			const char *segEnd = eatSegment(segBeg, end);
			if (!segEnd) {
				break;
			}
			r = segEnd;
			if (*segBeg != '<') {
				// find points for <wbr></wbr>
				const char *walker = segBeg;
				int numberNonSpaceChars = 0;
				while (walker < segEnd) {
					if (isspace(*walker)) {
						numberNonSpaceChars = 0;
					} else {
						numberNonSpaceChars++;
					}
					if (numberNonSpaceChars > kMaxNonSpaceChars) {
						numberNonSpaceChars = 1;
						[output appendBytes:pos length:walker - pos];
						pos = walker;
						static const char wbr[] = "<wbr></wbr>";
						[output appendBytes:wbr length:strlen(wbr)];
					}
					++walker;
				}
			} else {
				const char *attrBeg = NULL;
				const char *attrEnd = NULL;
				if (isTagWithAttr(segBeg, segEnd, "td", "class=\"default\"", &attrBeg, &attrEnd)) {
					state = ParseStateTdDefault;
				} else if (isTagWithAttr(segBeg, segEnd, "td", "class=\"title\"", &attrBeg, &attrEnd)) {
					state = ParseStateTdTitle;
				}
			}
		}
			goto more;
		case ParseStateTdDefault: {
			const char *attrBeg;
			const char *attrEnd;
			if (!skipThroughTagWithAttr(&r, end, "span", "id=score_", &attrBeg, &attrEnd)) {
				break;
			}
			const char *beginningOfCommentId = attrEnd;
			const char *endOfCommentId = eatToChar(beginningOfCommentId, end, '>');
			[currentCommentId release];
			currentCommentId = [[NSString alloc] initWithCString:beginningOfCommentId length:endOfCommentId - beginningOfCommentId];
			state = ParseStateTdDefaultAfterScore;
		}
			goto more;
		case ParseStateTdDefaultAfterScore: {
			if (!skipThroughCloser(&r, end, "span")) {
				break;
			}
			state = ParseStateTdDefaultAfterScoreSpan;
		}
			goto more;
		case ParseStateTdDefaultAfterScoreSpan: {
			const char *justBeforeCloser = skipThroughCloser(&r, end, "span");
			if (!justBeforeCloser) {
				break;
			}
			enum PostTrackerPostNovelty novelty = presumedThreadId ? [[PostTracker thePostTracker] getPostNovelty:currentCommentId inThread:presumedThreadId] : PostTrackerPostNoveltyNotNew;

			if (novelty == PostTrackerPostNoveltyNew) {
				[output appendBytes:pos length:justBeforeCloser - pos];
				NSString *boldNewNS = [NSString stringWithFormat:@" <b id=\"hnWebPost_%@\"><font color=\"#000000\">(new)</font></b><script>hnWebPosts.push(%@);</script>", currentCommentId, currentCommentId];
				const char *boldNew = [boldNewNS cStringUsingEncoding:NSUTF8StringEncoding];
				[output appendBytes:boldNew length:strlen(boldNew)];
				[output appendBytes:justBeforeCloser length:r - justBeforeCloser];
				pos = r;
			}

			state = ParseStateInContentTable;
		}
			goto more;
		case ParseStateTdTitle: {
			const char *attrBeg;
			const char *attrEnd;
			if (!skipThroughTagWithAttr(&r, end, "span", "id=score_", &attrBeg, &attrEnd)) {
				break;
			}
			const char *beginningOfThreadId = attrEnd;
			const char *endOfThreadId = eatToChar(beginningOfThreadId, end, '>');
			[presumedThreadId release];
			presumedThreadId = [[NSString alloc] initWithCString:beginningOfThreadId length:endOfThreadId - beginningOfThreadId];
			state = ParseStateTdTitleAfterScore;
		}
			goto more;
		case ParseStateTdTitleAfterScore: {
			const char *attrBeg;
			const char *attrEnd;
			if (!skipThroughTagWithAttr(&r, end, "a", "href=\"item?id=", &attrBeg, &attrEnd)) {
				break;
			}
			state = ParseStateTdTitleImmediatelyInCommentCount;
		}
			goto more;
		case ParseStateTdTitleImmediatelyInCommentCount: {
			const char *beginningOfCommentCount = r;
			const char *endOfCommentCount = eatToOneOf(r, end, " <");
			const char *endOfTextNode = eatToChar(r, end, '<');
			if (!endOfTextNode) {
				break;
			}
			r = endOfTextNode;
			// Now, how many comments do we have, and what's our thread id?
			NSData *commentCountStringData = [[NSData alloc] initWithBytes:beginningOfCommentCount length:endOfCommentCount - beginningOfCommentCount];
			NSString *commentCountString = [[NSString alloc] initWithData:commentCountStringData encoding:NSUTF8StringEncoding];
			[commentCountStringData release];
			NSInteger commentCount = [commentCountString integerValue];
			[commentCountString release];

			NSInteger knownComments = [[PostTracker thePostTracker] knownCommentsForThread:presumedThreadId];

			[output appendBytes:pos length:r - pos];
			pos = r;
			if (commentCount > knownComments) {
				NSString *s = [NSString stringWithFormat:(knownComments == 0 ? @" <b>(%d)</b>" : @" <b><font color=\"#000000\">(%d)</font></b>"), (commentCount - knownComments)];
				[output appendData:[s dataUsingEncoding:NSASCIIStringEncoding]];
			}
			[output appendData:[[NSString stringWithFormat:@"<script>hnWebThreadId = %@;</script>", presumedThreadId]
			                    dataUsingEncoding:NSASCIIStringEncoding]];
			state = ParseStateInContentTable;
		}
			goto more;
		default:
			r = end;
			break;
	}
	[output appendBytes:pos length:r - pos];
	*posRef = r;
}

- (NSData *)consumeData:(NSData *)data {
	[unparsedData appendData:data];
	NSMutableData *output = [[NSMutableData alloc] init];
	if ([unparsedData length] != 0) {
		const char *inputBytes = [unparsedData bytes];
		const char *inputBytesEnd = inputBytes + [unparsedData length];
		const char *pos = inputBytes;
		[self consume:output from:&pos to:inputBytesEnd];
		[unparsedData replaceBytesInRange:NSMakeRange(0, pos - inputBytes) withBytes:NULL length:0];
	}
	return [output autorelease];
}

- (void)dealloc {
	[presumedThreadId release];
	[currentCommentId release];
	[super dealloc];
}

@end
