#import "HNWebProtocol.h"
#import "BasicMuncher.h"
#import "NewsCSSMuncher.h"
#import "PostTracker.h"

NSString *HackerNewsScheme = @"hn";

@implementation HNWebProtocol

+ (BOOL)canInitWithRequest:(NSURLRequest *)request {
	return [[[request URL] scheme] caseInsensitiveCompare:HackerNewsScheme] == NSOrderedSame;
}
+ (NSURLRequest *)canonicalRequestForRequest:(NSURLRequest *)request {
	return request;
}
+ (NSURL *)httpURLFromHnURL:(NSURL *)url {
	NSString *hnColonSlashSlash = [HackerNewsScheme stringByAppendingString:@"://"];
	return [NSURL URLWithString:[[url description] stringByReplacingOccurrencesOfString:hnColonSlashSlash
	                                                                         withString:@"http://"
	                                                                            options:NSCaseInsensitiveSearch
	                                                                              range:NSMakeRange(0, [hnColonSlashSlash length])]];
}
- (void)startLoading {
	id<NSURLProtocolClient> client = [self client];
	NSURLRequest *request = [self request];
	NSURL *requestURL = [request URL];
	if ([@"/hnweb.post.viewed" isEqualToString:[requestURL path]]) {
		// HOSTILE DATA
		NSString *q = [requestURL query];
		if (q != nil) {
			NSArray *qParts = [q componentsSeparatedByString:@"_"];
			if ([qParts count] == 2) {
				// HOSTILE DATA
				int threadNumber = [[qParts objectAtIndex:0] intValue];
				int postNumber = [[qParts objectAtIndex:1] intValue];

				if (postNumber > 0 && threadNumber > 0) {
					NSLog(@"Noticing post id %d in thread %d with query %@", postNumber, threadNumber, q);
					[[PostTracker thePostTracker] noticePost:[NSString stringWithFormat:@"%d", postNumber]
					                                inThread:[NSString stringWithFormat:@"%d", threadNumber]];
					[[self client] URLProtocolDidFinishLoading:self];
					return;
				}
			}
		}
		NSLog(@"Bad query string: %@", q);
		// close response
		return;
	}



	NSLog(@"startLoading: %@", requestURL);
	NSURL *httpURL = [NSURL URLWithString:[[requestURL description] stringByReplacingCharactersInRange:NSMakeRange(0, 2) withString:@"http"]];


	NSMutableURLRequest *mutableRequest = [request mutableCopy];
	[mutableRequest setURL:httpURL];
	muncher = [@"/news.css" isEqualToString:[requestURL path]] ? [[NewsCSSMuncher alloc] init] : [[BasicMuncher alloc] init];
	theConnection = [[NSURLConnection alloc] initWithRequest:mutableRequest
	                                                delegate:self];
	[mutableRequest release];
	if (!theConnection) {
		[client URLProtocol:self
		   didFailWithError:[NSError errorWithDomain:NSURLErrorDomain
		                                        code:NSURLErrorUnknown
		                                    userInfo:nil]];
	}
}
- (void)stopLoading {
	[theConnection cancel];
	[muncher release];
	muncher = nil;
	[theConnection release];
	theConnection = nil;
}


#pragma mark URLConnection delegate methods

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response {
	NSLog(@"didReceiveResponse");
	// update state
	[muncher reset];
	// inform client
	[[self client] URLProtocol:self didReceiveResponse:response cacheStoragePolicy:NSURLCacheStorageNotAllowed];
}
- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
	NSUInteger n = [data length];
	NSLog(@"didReceiveData with length %i", n);
	// update state
	NSData *dataToSend = [muncher consumeData:data];
	// inform client
	[[self client] URLProtocol:self didLoadData:dataToSend];
}
- (void)connection:(NSURLConnection *)connection
  didFailWithError:(NSError *)error {
	NSLog(@"didFailWithError");
	// update state
	[muncher reset];
	// inform client
	[[self client] URLProtocol:self didFailWithError:error];
}
- (void)connectionDidFinishLoading:(NSURLConnection *)connection {
	NSLog(@"connectionDidFinishLoading");
	// send leftover data
	NSData *data = [muncher finish];
	if (0 < [data length]) {
		// todo investigate whether this extra copyage is necessary.
		[[self client] URLProtocol:self didLoadData:data];
	}
	[[self client] URLProtocolDidFinishLoading:self];
}


- (void)dealloc {
	[theConnection release];
	[muncher release];
	[super dealloc];
}


@end
