#import "HNWebViewController.h"
#import "HNWebProtocol.h"


#import <arpa/inet.h>
#import <SystemConfiguration/SystemConfiguration.h>

@implementation HNWebViewController

@synthesize webView, toolbar, backButton, forwardButton, refreshButton, stopButton;


/*
// The designated initializer. Override to perform setup that is required before the view is loaded.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
        // Custom initialization
    }
    return self;
}
*/

/*
// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView {
}
*/



// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
	[NSURLProtocol registerClass:[HNWebProtocol class]];
	NSString *urlString = [NSString stringWithFormat:@"%@://news.ycombinator.com/", HackerNewsScheme];
	[self considerReachability];
	[webView loadRequest:[NSURLRequest requestWithURL:[NSURL URLWithString:urlString]]];
	[super viewDidLoad];
}
- (IBAction)backButtonPressed:(id)sender {
	NSLog(@"back button pressed!");
	[self considerReachability];
	[webView goBack];
	[self refreshButtonEnablement];
}
- (IBAction)forwardButtonPressed:(id)sender {
	NSLog(@"forward button pressed!");
	[self considerReachability];
	[webView goForward];
	[self refreshButtonEnablement];
}
- (IBAction)refreshButtonPressed:(id)sender {
	NSLog(@"refresh button pressed!");
	[self considerReachability];
	[webView reload];
	[self refreshButtonEnablement];
}
- (IBAction)stopButtonPressed:(id)sender {
	NSLog(@"stop button pressed!");
	[self considerReachability];
	[webView stopLoading];
	[self refreshButtonEnablement];

}
- (IBAction)actionButtonPressed:(id)sender {
	NSLog(@"action button pressed!");
	[self considerReachability];
	[[UIApplication sharedApplication] openURL:[HNWebProtocol httpURLFromHnURL:[[webView request] URL]]];
}
- (IBAction)pageDownButtonPressed:(id)sender {
	NSLog(@"pageDown button pressed!");
	[self considerReachability];
	[webView stringByEvaluatingJavaScriptFromString:@"window.scrollBy(0, window.innerHeight-60); hnWebCheck();"];
}
- (IBAction)pageUpButtonPressed:(id)sender {
	NSLog(@"pageUp button pressed!");
	[self considerReachability];
	[webView stringByEvaluatingJavaScriptFromString:@"hnWebCheck(); window.scrollBy(0, -(window.innerHeight-60));"];
}

- (void)considerReachability {
	if (![self connectedToNetwork]) {
		UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"NO NO NO"
		                                                message:@"No Internet Connection!"
		                                               delegate:nil
		                                      cancelButtonTitle:@"Okay"
		                                      otherButtonTitles:nil];
		[alert show];
		[alert release];
	}
}

#pragma mark -
#pragma mark UIWebViewDelegate methods

- (void)webViewDidStartLoad:(UIWebView *)webView {
	[self refreshButtonEnablement];
	[[UIApplication sharedApplication] setNetworkActivityIndicatorVisible:YES];
	[self setRefreshMode: FALSE];
}
- (void)webViewDidFinishLoad:(UIWebView *)webView {
	[self refreshButtonEnablement];
	[[UIApplication sharedApplication] setNetworkActivityIndicatorVisible:NO];
	[self setRefreshMode: TRUE];
}
- (void)webView:(UIWebView *)webView didFailLoadWithError:(NSError *)error {
	[self refreshButtonEnablement];
	[[UIApplication sharedApplication] setNetworkActivityIndicatorVisible:NO];
	[self setRefreshMode: TRUE];
}

#pragma mark -
#pragma mark Other methods

- (BOOL) connectedToNetwork
{
	struct sockaddr_in zero;
	memset(&zero, 0, sizeof(zero));
	zero.sin_len = sizeof(zero);
	zero.sin_family = AF_INET;
	// Recover reachability flags
	SCNetworkReachabilityRef defaultRouteReachability = SCNetworkReachabilityCreateWithAddress(NULL, (struct sockaddr*)&zero);
	SCNetworkReachabilityFlags flags;
	BOOL didRetrieveFlags = SCNetworkReachabilityGetFlags(defaultRouteReachability, &flags);
	CFRelease(defaultRouteReachability);
	if (!didRetrieveFlags)
	{
		NSLog(@"Error. Could not recover network reachability flags");
		return 0;
	}
	BOOL isReachable = flags & kSCNetworkFlagsReachable;
	BOOL needsConnection = flags & kSCNetworkFlagsConnectionRequired;
	return (isReachable && !needsConnection) ? YES : NO;
}

- (void)refreshButtonEnablement {
	backButton.enabled = [webView canGoBack];
	forwardButton.enabled = [webView canGoForward];
}
- (void)setRefreshMode:(BOOL)refreshMode {
	NSMutableArray *newItems = [[NSMutableArray alloc] init];
	for (UIBarButtonItem *item in toolbar.items) {
		if (item == stopButton || item == refreshButton) {
			[newItems addObject:(refreshMode ? refreshButton : stopButton)];
		} else {
			[newItems addObject:item];
		}
	}
	toolbar.items = newItems;
	[newItems release];
}


/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
	// Return YES for supported orientations
	return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
	[super didReceiveMemoryWarning];

	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (void)dealloc {
	webView.delegate = nil;
	[webView release];
	[toolbar release];
	[backButton release];
	[forwardButton release];
	[refreshButton release];
	[stopButton release];
	[super dealloc];
}

@end
